# Generated by Django 3.2.8 on 2021-12-21 11:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0005_auto_20201013_0310'),
        ('corptools', '0059_invtypematerials_met_type'),
        ('eveonline', '0014_auto_20210105_1413'),
        ('moons', '0016_invoicerecord_base_ref'),
    ]

    operations = [
        migrations.CreateModel(
            name='MoonRental',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('note', models.TextField()),
                ('price', models.IntegerField(default=100000000)),
                ('start_date', models.DateTimeField()),
                ('end_date', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('contact', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='eveonline.evecharacter')),
                ('corporation', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='eveonline.evecorporationinfo')),
                ('last_invoice', models.ForeignKey(blank=True, default=None, null=True,
                 on_delete=django.db.models.deletion.SET_NULL, to='invoices.invoice')),
                ('moon', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.mapsystemmoon')),
            ],
        ),
    ]
