.padded-label{margin:5px}@-webkit-keyframes spin{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}to{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}@keyframes spin{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}to{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}.glyphicon-refresh-animate{-animation:spin 2s infinite linear;-ms-animation:spin 2s infinite linear;-webkit-animation:spin 2s linear infinite;-moz-animation:spin 2s infinite linear}.flex-container{display:flex;align-items:center;justify-content:center;align-content:center;flex-wrap:wrap;flex-direction:row}.spinner-size{height:30px;width:30px;margin:200px}.error-size{width:100px;height:100px;margin:200px 200px 15px}.text-margin{margin-bottom:185px}@-webkit-keyframes shake{10%,90%{-webkit-transform:translate3d(-1px,0,0);transform:translate3d(-1px,0,0)}20%,80%{-webkit-transform:translate3d(2px,0,0);transform:translate3d(2px,0,0)}30%,50%,70%{-webkit-transform:translate3d(-4px,0,0);transform:translate3d(-4px,0,0)}40%,60%{-webkit-transform:translate3d(4px,0,0);transform:translate3d(4px,0,0)}}@keyframes shake{10%,90%{-webkit-transform:translate3d(-1px,0,0);transform:translate3d(-1px,0,0)}20%,80%{-webkit-transform:translate3d(2px,0,0);transform:translate3d(2px,0,0)}30%,50%,70%{-webkit-transform:translate3d(-4px,0,0);transform:translate3d(-4px,0,0)}40%,60%{-webkit-transform:translate3d(4px,0,0);transform:translate3d(4px,0,0)}}.error-anim{-webkit-animation:shake .82s cubic-bezier(.36,.07,.19,.97) both;animation:shake .82s cubic-bezier(.36,.07,.19,.97) both;-webkit-animation-iteration-count:infinite;animation-iteration-count:infinite}.jackpot{-webkit-animation:wobble 1s ease infinite;animation:wobble 1s ease infinite}@-webkit-keyframes wobble{0%,to{-webkit-transform:translateX(0);transform:translateX(0);-webkit-transform-origin:50% 50%;transform-origin:50% 50%}15%{-webkit-transform:translateX(-16px) rotate(-5deg);transform:translateX(-16px) rotate(-5deg)}30%{-webkit-transform:translateX(8px) rotate(5deg);transform:translateX(8px) rotate(5deg)}45%{-webkit-transform:translateX(-8px) rotate(-2.77778deg);transform:translateX(-8px) rotate(-2.77778deg)}60%{-webkit-transform:translateX(4.84848px) rotate(1.66667deg);transform:translateX(4.84848px) rotate(1.66667deg)}75%{-webkit-transform:translateX(-2.90909px) rotate(-1deg);transform:translateX(-2.90909px) rotate(-1deg)}}@keyframes wobble{0%,to{-webkit-transform:translateX(0);transform:translateX(0);-webkit-transform-origin:50% 50%;transform-origin:50% 50%}15%{-webkit-transform:translateX(-16px) rotate(-5deg);transform:translateX(-16px) rotate(-5deg)}30%{-webkit-transform:translateX(8px) rotate(5deg);transform:translateX(8px) rotate(5deg)}45%{-webkit-transform:translateX(-8px) rotate(-2.77778deg);transform:translateX(-8px) rotate(-2.77778deg)}60%{-webkit-transform:translateX(4.84848px) rotate(1.66667deg);transform:translateX(4.84848px) rotate(1.66667deg)}75%{-webkit-transform:translateX(-2.90909px) rotate(-1deg);transform:translateX(-2.90909px) rotate(-1deg)}}
/*# sourceMappingURL=main.ba2c4f57.chunk.css.map */